﻿local Waterfall = AceLibrary:HasInstance("Waterfall-1.0") and AceLibrary("Waterfall-1.0");

BeneCast.GetSnapToOption = function()

	if BeneCastSetup.SnapTo == nil then
		BeneCastSetup.SnapTo = 'BeneCast';
	end
	return BeneCast.SnapTo[BeneCastSetup.SnapTo].name;
	
end

BeneCast.SetSnapToOption = function(addon_name)

	for addonname, addon in pairs(wv.SnapToFound) do
		if addon.name == addon_name then
			BeneCastSetup.SnapTo = addonname;
			BeneCast.SnapTo:ApplyConfigTable();
			BeneCast.SnapTo:SnapPanels();
			do break end;
		end
	end
	
end

BeneCast.GenerateSnapToOptions = function()

	local snaptooptions = {};
	for addonname, addon in pairs(wv.SnapToFound) do
		table.insert(snaptooptions,addon.name);
	end
	return snaptooptions;
	
end

local options = {
    name = 'BeneCast',
    handler = BeneCast,
    type = 'group',
    args = {
        win = {
            type = 'execute',
            name = L['Settings'],
            func = function()
				if Waterfall then
					Waterfall:Open('BeneCast')
				else
					RatingBuster:Print(L["Waterfall-1.0 is required to access the GUI."])
				end
            end,
        },
        snapto = {
			type = 'select',
			name = L['Snap To Options'],
			values = function()
				local snaptooptions = {};
				for addonname, addon in pairs(wv.SnapToFound) do
					table.insert(snaptooptions,addon.name);
				end
				return snaptooptions;
			end,
			get = function()
				if BeneCastSetup.SnapTo == nil then
					BeneCastSetup.SnapTo = 'BeneCast';
				end
				return BeneCast.SnapTo[BeneCastSetup.SnapTo].name;
			end,
			set = function(addon_name)
				for addonname, addon in pairs(wv.SnapToFound) do
					if addon.name == addon_name then
						BeneCastSetup.SnapTo = addonname;
						BeneCast.SnapTo:ApplyConfigTable();
						BeneCast.SnapTo:SnapPanels();
						do break end;
					end
				end
			end,
        },
        debug = {
			type = 'toggle',
			name = 'Debug',
			get = function()
				return BeneCastSetup.Debug;
			end,
			set = function(val)
				BeneCastSetup.Debug = val;
			end,
        },
        hideminimapbutton = {
			type = 'toggle',
			name = L['Hide the minimap button'],
			get = function()
				return BeneCastSetup.HideMinimapButton;
			end,
			set = function(val)
				BeneCastSetup.HideMinimapButton = val;
				if MinimapButton then
					if val then
						MinimapButton:Hide();
					else
						MinimapButton:Show();
					end
				end
			end,
        },
        profiles = {
			type = 'group',
			name = 'Profiles',
			args = {
				settings = {
					type = 'group',
					name = L['Setup'],
					args {
						-- TODO: add stuff like default orientation and buttonsize and such
					},
				},
				player = {
					type = 'group',
					name = L['Player'],
					args = { }, -- This will be filled as the spells load
				},
				pet = {
					type = 'group',
					name = L['Pet'],
					args = { }, -- This will be filled as the spells load
				},
				party = {
					type = 'group',
					name = L['Group Members'],
					args = { }, -- This will be filled as the spells load
				},
				partypet = {
					type = 'group',
					name = L['Group Pets'],
					args = { }, -- This will be filled as the spells load
				},
				other = {
					type = 'group',
					name = L['Non-Group Members'],
					args = { }, -- This will be filled as the spells load
				},
			},
        },
    },
};

-- Function to generate an optiontable from a spell/item/...-name and a config
-- For now this is a toggle, but later it can be anything
function BeneCast:ActionToConfig(groupname, configname)
	local spelloption = {
		type = 'toggle',
		name = groupname,
		passValue = {
			group = groupname,
			config = configname,
		},
		get = function(param)
			if profile.Config[param.config][param.group] == nil then
				profile.Config[param.config][param.group] = { enabled = false };
			end
			local setting = profile.Config[param.config][param.group];
			return setting.enabled;
		end,
		set = function(param, val)
			if profile.Config[param.config][param.group] == nil then
				profile.Config[param.config][param.group] = { enabled = false };
			end
			local setting = profile.Config[param.config][param.group];
			setting.enabled = val;
		end,
	};
	return spelloption;
end

-- Function to generate entire spellconfig by the known spells-list
function BeneCast:GenerateSpellConfigFor(configname)
	local optionstable = options.args.profiles.args[configname].args;
	local propergrp;
	_ClearTable(optionstable, true);
	
	-- First add assist and focus
	if configname ~= 'player' then
		optionstable['assist'] = self:ActionToConfig('assist', configname);
	end
	optionstable['focus'] = self:ActionToConfig('focus', configname);
	-- Then do spells
	for group, spell in pairs(wv.Spells) do
		if spell.target and BeneCast.Constants.TargetTypes[spell.target][configname] then
			propergrp = string.gsub(group, ' ', '');
			optionstable[propergrp] = self:ActionToConfig(group, configname);
		end
	end
	-- TODO: Items and macro's once the parsing of that is done
end

-- Function to do the above for ALL configs
function BeneCast:GenerateAllSpellConfigs()
	for i, configname in pairs(BeneCast.Constants.Configs) do
		self:GenerateSpellConfigFor(configname);
	end
end